@@ -1,7 +1,7 @@
 import logging
 import tensorflow as tf
-from eight_mile.utils import get_version, exporter
-from eight_mile.optz import create_lr_scheduler, register_lr_scheduler
+from eight_mile.utils import get_version, exporter, register
+from eight_mile.optz import create_lr_scheduler, register_lr_scheduler, MEAD_LAYERS_LR_SCHEDULERS
 from eight_mile.optz import (
     LearningRateScheduler,
     ConstantScheduler,
@@ -11,266 +11,310 @@ from eight_mile.optz import (
     PiecewiseDecayScheduler,
     ZarembaDecayScheduler,
     CosineDecayScheduler,
-    InverseTimeDecayScheduler,
-    ExponentialDecayScheduler,
-    CompositeLRScheduler,
 )
 
 logger = logging.getLogger("mead.layers")
-__all__ = []
-export = exporter(__all__)
+
+
+class ConstantSchedulerTensorFlow1:
+    def __init__(self, **kwargs):
+        pass
+
+    def __call__(self, lr, global_step):
+        return tf.identity(lr, name="lr")
+
+    def __str__(self):
+        return type(self).__name__ + "()"
+
+
+class WarmupLinearSchedulerTensorFlow1(WarmupLearningRateScheduler):
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+
+    def __call__(self, lr, global_step):
+        return tf.identity(
+            tf.minimum(1.0, tf.cast(global_step / self.warmup_steps, dtype=tf.float32)) * lr, name="lr"
+        )
+
+    def __str__(self):
+        return type(self).__name__ + "()"
+
+
+class CyclicLRSchedulerTensorFlow1:
+    def __init__(self, max_lr=1e-2, decay_steps=1000, **kwargs):
+        super().__init__()
+        self.max_lr = max_lr
+        self.decay_steps = decay_steps
+
+    def __call__(self, lr, global_step):
+        gs_f = tf.cast(global_step, tf.float32)
+        cycle = tf.floor(1.0 + gs_f / (2.0 * self.decay_steps))
+        x = tf.abs(gs_f / self.decay_steps - 2.0 * cycle + 1.0)
+        clr = lr + (self.max_lr - lr) * tf.maximum(0.0, 1.0 - x)
+        return tf.identity(clr, name="lr")
+
+    def __str__(self):
+        return type(self).__name__ + "()"
+
+
+class SGDRSchedulerTensorFlow1:
+    def __init__(self, first_decay_steps=1000, **kwargs):
+        super().__init__()
+        self.first_decay_steps = first_decay_steps
+
+    def __call__(self, lr, global_step):
+        return tf.identity(
+            tf.train.cosine_decay_restarts(lr, global_step, first_decay_steps=self.first_decay_steps), name="lr"
+        )
+
+    def __str__(self):
+        return type(self).__name__ + "()"
+
+
+class CompositeLRSchedulerTensorFlow1:
+    def __init__(self, warm=None, rest=None, **kwargs):
+        self.warm = warm
+        self.rest = rest
+
+    def __call__(self, lr, global_step):
+        warm_tensor = self.warm(lr, global_step)
+
+        def call_warm():
+            return warm_tensor
+
+        rest_step = tf.subtract(global_step, tf.compat.v1.constant(self.warm.warmup_steps, dtype=global_step.dtype))
+        rest_tensor = self.rest(lr, rest_step)
+
+        def call_rest():
+            return rest_tensor
+
+        return tf.identity(tf.cond(global_step < self.warm.warmup_steps, call_warm, call_rest), name="lr")
+
+    def __str__(self):
+        return "LRScheduler({}, {})".format(self.warm, self.rest)
+
+
+class PiecewiseDecaySchedulerTensorFlow1:
+    def __init__(self, boundaries=None, values=None, **kwargs):
+        super().__init__()
+        self.boundaries = boundaries
+        self.values = values
+
+    def __call__(self, lr, global_step):
+        return tf.identity(tf.compat.v1.train.piecewise_constant(global_step, self.boundaries, self.values), name="lr")
+
+    def __str__(self):
+        return type(self).__name__ + "()"
+
+
+class InverseTimeDecaySchedulerTensorFlow1:
+    def __init__(self, decay_steps=16000, decay_rate=0.05, staircase=False, **kwargs):
+        super().__init__()
+        self.decay_steps = decay_steps
+        self.decay_rate = decay_rate
+        self.staircase = staircase
+
+    def __call__(self, lr, global_step):
+        return tf.identity(
+            tf.train.inverse_time_decay(
+                lr, global_step, self.decay_steps, self.decay_rate, staircase=self.staircase
+            ),
+            name="lr",
+        )
+
+    def __str__(self):
+        return type(self).__name__ + "()"
+
+
+class ExponentialDecaySchedulerTensorFlow1:
+    def __init__(self, decay_steps=16000, decay_rate=0.5, staircase=False, **kwargs):
+        self.decay_steps = decay_steps
+        self.decay_rate = decay_rate
+        self.staircase = staircase
+
+    def __call__(self, lr, global_step):
+        return tf.identity(
+            tf.train.exponential_decay(
+                lr, global_step, self.decay_steps, self.decay_rate, staircase=self.staircase
+            ),
+            name="lr",
+        )
+
+    def __str__(self):
+        return type(self).__name__ + "()"
+
+
+class ZarembaDecaySchedulerTensorFlow1(PiecewiseDecaySchedulerTensorFlow1):
+    """Utility only, just to simplify the JSON"""
+
+    def __init__(self, boundaries=None, decay_rate=None, **kwargs):
+        lr = float(kwargs.get("lr", kwargs.get("eta", 1.0)))
+        values = [lr / (float(decay_rate) ** i) for i in range(len(boundaries) + 1)]
+        super().__init__(boundaries=boundaries, values=values)
+
+    def __str__(self):
+        return type(self).__name__ + "()"
+
+
+class ConstantSchedulerTensorFlow2(LearningRateScheduler, tf.keras.optimizers.schedules.LearningRateSchedule):
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+
+    def __call__(self, global_step):
+        return self.lr
+
+    def __str__(self):
+        return type(self).__name__ + "()"
+
+
+class WarmupLinearSchedulerTensorFlow2(
+    WarmupLearningRateScheduler, tf.keras.optimizers.schedules.LearningRateSchedule
+):
+    def __init__(self, warmup_steps=16000, **kwargs):
+        lr = float(kwargs.get("lr", kwargs.get("eta", 1.0)))
+        kwargs["lr"] = lr
+        super().__init__(warmup_steps=warmup_steps, **kwargs)
+
+    def __call__(self, global_step):
+        return tf.minimum(1.0, global_step / float(self.warmup_steps)) * self.lr
+
+    def __str__(self):
+        return type(self).__name__ + "()"
+
+
+class CyclicLRSchedulerTensorFlow2(LearningRateScheduler, tf.keras.optimizers.schedules.LearningRateSchedule):
+    def __init__(self, max_lr=1e-2, decay_steps=1000, **kwargs):
+        lr = float(kwargs.get("lr", kwargs.get("eta", 1.0)))
+        kwargs["lr"] = lr
+        super().__init__(**kwargs)
+        self.max_lr = max_lr
+        self.decay_steps = decay_steps
+
+    def __call__(self, global_step):
+        gs_f = tf.cast(global_step, tf.float32)
+        cycle = tf.floor(1.0 + gs_f / (2.0 * self.decay_steps))
+        x = tf.abs(gs_f / self.decay_steps - 2.0 * cycle + 1.0)
+        clr = self.lr + (self.max_lr - self.lr) * tf.maximum(0.0, 1.0 - x)
+        return tf.identity(clr, name="lr")
+
+    def __str__(self):
+        return type(self).__name__ + "()"
+
+
+class SGDRSchedulerTensorFlow2(LearningRateScheduler, tf.keras.optimizers.schedules.LearningRateSchedule):
+    def __init__(self, first_decay_steps=1000, **kwargs):
+        super().__init__(**kwargs)
+        self.first_decay_steps = first_decay_steps
+
+    def __call__(self, global_step):
+        return tf.identity(
+            tf.compat.v1.train.cosine_decay_restarts(
+                self.lr, global_step, first_decay_steps=self.first_decay_steps
+            ),
+            name="lr",
+        )
+
+    def __str__(self):
+        return type(self).__name__ + "()"
+
+
+class ZarembaDecaySchedulerTensorFlow2(tf.keras.optimizers.schedules.PiecewiseConstantDecay):
+    """Utility only, just to simplify the JSON"""
+
+    def __init__(self, boundaries=None, decay_rate=None, **kwargs):
+        lr = float(kwargs.get("lr", kwargs.get("eta", 1.0)))
+        values = [lr / (float(decay_rate) ** i) for i in range(len(boundaries) + 1)]
+        super().__init__(boundaries, values, kwargs.get("name"))
+
+
+class CompositeLRSchedulerTensorFlow2(tf.keras.optimizers.schedules.LearningRateSchedule):
+    def __init__(self, warm=None, rest=None, **kwargs):
+        self.warm = warm
+        self.rest = rest
+
+    def __call__(self, global_step):
+        warm_tensor = self.warm(global_step)
+
+        def call_warm():
+            return warm_tensor
+
+        rest_step = global_step - self.warm.warmup_steps
+        rest_tensor = self.rest(rest_step)
+
+        def call_rest():
+            return rest_tensor
+
+        new_lr = tf.cond(global_step < self.warm.warmup_steps, call_warm, call_rest)
+        tf.summary.scalar(name='lr', data=new_lr, step=tf.cast(global_step, tf.int64))
+        return new_lr
+
+    def __str__(self):
+        return "LRScheduler({}, {})".format(self.warm, self.rest)
+
+
+class PiecewiseDecaySchedulerTensorFlow2(tf.keras.optimizers.schedules.PiecewiseConstantDecay):
+    def __init__(self, boundaries, values, **kwargs):
+        super().__init__(boundaries, values)
+
+
+class InverseTimeDecaySchedulerTensorFlow2(tf.keras.optimizers.schedules.InverseTimeDecay):
+    def __init__(self, decay_steps=16000, decay_rate=0.05, staircase=False, **kwargs):
+        lr = kwargs.get("lr", kwargs.get("eta", 0.01))
+        super().__init__(lr, decay_steps, decay_rate, staircase, kwargs.get("name"))
+
+
+class ExponentialDecaySchedulerTensorFlow2(tf.keras.optimizers.schedules.ExponentialDecay):
+    def __init__(self, decay_steps=16000, decay_rate=0.5, staircase=False, **kwargs):
+        lr = kwargs.get("lr", kwargs.get("eta", 0.01))
+        super().__init__(lr, decay_steps, decay_rate, staircase, kwargs.get("name"))
+
+
+class CosineDecaySchedulerTensorFlow(CosineDecayScheduler, tf.keras.optimizers.schedules.LearningRateSchedule):
+    def __init__(self, decay_steps=16000, alpha=0.0, **kwargs):
+        kwargs['lr'] = kwargs.get("lr", kwargs.get("eta", 0.01))
+        super().__init__(decay_steps, alpha, **kwargs)
+
+    def __call__(self, global_step):
+        global_step = tf.math.minimum(global_step, self.decay_steps)
+        cosine_decay = 0.5 * (1.0 + tf.cos(3.14159265 * global_step / self.decay_steps))
+        decayed = (1 - self.alpha) * cosine_decay + self.alpha
+        return self.lr * decayed
 
 
 if not tf.executing_eagerly():
 
-    @register_lr_scheduler("default")
-    class ConstantSchedulerTensorFlow1:
-        def __init__(self, **kwargs):
-            pass
-
-        def __call__(self, lr, global_step):
-            return tf.identity(lr, name="lr")
-
-        def __str__(self):
-            return type(self).__name__ + "()"
-
-    @register_lr_scheduler("warmup_linear")
-    class WarmupLinearSchedulerTensorFlow1(WarmupLearningRateScheduler):
-        def __init__(self, **kwargs):
-            super().__init__(**kwargs)
-
-        def __call__(self, lr, global_step):
-            return tf.identity(
-                tf.minimum(1.0, tf.cast(global_step / self.warmup_steps, dtype=tf.float32)) * lr, name="lr"
-            )
-
-        def __str__(self):
-            return type(self).__name__ + "()"
-
-    @register_lr_scheduler("clr")
-    class CyclicLRSchedulerTensorFlow1:
-        def __init__(self, max_lr=1e-2, decay_steps=1000, **kwargs):
-            super().__init__()
-            self.max_lr = max_lr
-            self.decay_steps = decay_steps
-
-        def __call__(self, lr, global_step):
-            gs_f = tf.cast(global_step, tf.float32)
-            cycle = tf.floor(1.0 + gs_f / (2.0 * self.decay_steps))
-            x = tf.abs(gs_f / self.decay_steps - 2.0 * cycle + 1.0)
-            clr = lr + (self.max_lr - lr) * tf.maximum(0.0, 1.0 - x)
-            return tf.identity(clr, name="lr")
-
-        def __str__(self):
-            return type(self).__name__ + "()"
-
-    @register_lr_scheduler("sgdr")
-    class SGDRSchedulerTensorFlow1:
-        def __init__(self, first_decay_steps=1000, **kwargs):
-            super().__init__()
-            self.first_decay_steps = first_decay_steps
-
-        def __call__(self, lr, global_step):
-            return tf.identity(
-                tf.train.cosine_decay_restarts(lr, global_step, first_decay_steps=self.first_decay_steps), name="lr"
-            )
-
-        def __str__(self):
-            return type(self).__name__ + "()"
-
-    @register_lr_scheduler("composite")
-    class CompositeLRSchedulerTensorFlow1:
-        def __init__(self, warm=None, rest=None, **kwargs):
-            self.warm = warm
-            self.rest = rest
-
-        def __call__(self, lr, global_step):
-            warm_tensor = self.warm(lr, global_step)
-
-            def call_warm():
-                return warm_tensor
-
-            rest_step = tf.subtract(global_step, tf.compat.v1.constant(self.warm.warmup_steps, dtype=global_step.dtype))
-            rest_tensor = self.rest(lr, rest_step)
-
-            def call_rest():
-                return rest_tensor
-
-            return tf.identity(tf.cond(global_step < self.warm.warmup_steps, call_warm, call_rest), name="lr")
-
-        def __str__(self):
-            return "LRScheduler({}, {})".format(self.warm, self.rest)
-
-    @register_lr_scheduler("piecewise")
-    class PiecewiseDecaySchedulerTensorFlow:
-        def __init__(self, boundaries=None, values=None, **kwargs):
-            super().__init__()
-            self.boundaries = boundaries
-            self.values = values
-
-        def __call__(self, lr, global_step):
-            return tf.identity(tf.compat.v1.train.piecewise_constant(global_step, self.boundaries, self.values), name="lr")
-
-        def __str__(self):
-            return type(self).__name__ + "()"
-
-    @register_lr_scheduler("invtime")
-    class InverseTimeDecaySchedulerTensorFlow:
-        def __init__(self, decay_steps=16000, decay_rate=0.05, staircase=False, **kwargs):
-            super().__init__()
-            self.decay_steps = decay_steps
-            self.decay_rate = decay_rate
-            self.staircase = staircase
-
-        def __call__(self, lr, global_step):
-            return tf.identity(
-                tf.train.inverse_time_decay(
-                    lr, global_step, self.decay_steps, self.decay_rate, staircase=self.staircase
-                ),
-                name="lr",
-            )
-
-        def __str__(self):
-            return type(self).__name__ + "()"
-
-    @register_lr_scheduler("exponential")
-    class ExponentialDecaySchedulerTensorFlow(object):
-        def __init__(self, decay_steps=16000, decay_rate=0.5, staircase=False, **kwargs):
-            self.decay_steps = decay_steps
-            self.decay_rate = decay_rate
-            self.staircase = staircase
-
-        def __call__(self, lr, global_step):
-            return tf.identity(
-                tf.train.exponential_decay(
-                    lr, global_step, self.decay_steps, self.decay_rate, staircase=self.staircase
-                ),
-                name="lr",
-            )
-
-        def __str__(self):
-            return type(self).__name__ + "()"
-
-    @register_lr_scheduler("zaremba")
-    class ZarembaDecaySchedulerTensorFlow1(PiecewiseDecaySchedulerTensorFlow):
-        """Utility only, just to simplify the JSON"""
-
-        def __init__(self, boundaries=None, decay_rate=None, **kwargs):
-            lr = float(kwargs.get("lr", kwargs.get("eta", 1.0)))
-            values = [lr / (float(decay_rate) ** i) for i in range(len(boundaries) + 1)]
-            super().__init__(boundaries=boundaries, values=values)
-
-        def __str__(self):
-            return type(self).__name__ + "()"
-
-
+    ConstantSchedulerTensorFlow = ConstantSchedulerTensorFlow1
+    WarmupLinearSchedulerTensorFlow = WarmupLinearSchedulerTensorFlow1
+    CyclicLRSchedulerTensorFlow = CyclicLRSchedulerTensorFlow1
+    SGDRSchedulerTensorFlow = SGDRSchedulerTensorFlow1
+    CompositeLRSchedulerTensorFlow = CompositeLRSchedulerTensorFlow1
+    PiecewiseDecaySchedulerTensorFlow = PiecewiseDecaySchedulerTensorFlow1
+    ExponentialDecaySchedulerTensorFlow = ExponentialDecaySchedulerTensorFlow1
+    InverseTimeDecaySchedulerTensorFlow = InverseTimeDecaySchedulerTensorFlow1
+    ZarembaDecaySchedulerTensorFlow = ZarembaDecaySchedulerTensorFlow1
 else:
+    ConstantSchedulerTensorFlow = ConstantSchedulerTensorFlow2
+    WarmupLinearSchedulerTensorFlow = WarmupLinearSchedulerTensorFlow2
+    CyclicLRSchedulerTensorFlow = CyclicLRSchedulerTensorFlow2
+    SGDRSchedulerTensorFlow = SGDRSchedulerTensorFlow2
+    CompositeLRSchedulerTensorFlow = CompositeLRSchedulerTensorFlow2
+    PiecewiseDecaySchedulerTensorFlow = PiecewiseDecaySchedulerTensorFlow2
+    ExponentialDecaySchedulerTensorFlow = ExponentialDecaySchedulerTensorFlow2
+    InverseTimeDecaySchedulerTensorFlow = InverseTimeDecaySchedulerTensorFlow2
+    ZarembaDecaySchedulerTensorFlow = ZarembaDecaySchedulerTensorFlow2
 
-    @register_lr_scheduler("default")
-    class ConstantSchedulerTensorFlow2(LearningRateScheduler, tf.keras.optimizers.schedules.LearningRateSchedule):
-        def __init__(self, **kwargs):
-            super().__init__(**kwargs)
+register(ConstantSchedulerTensorFlow, MEAD_LAYERS_LR_SCHEDULERS, "default", "lr_scheduler")
+register(WarmupLinearSchedulerTensorFlow, MEAD_LAYERS_LR_SCHEDULERS, "warmup_linear", "lr_scheduler")
+register(CyclicLRSchedulerTensorFlow, MEAD_LAYERS_LR_SCHEDULERS, "clr", "lr_scheduler")
+register(SGDRSchedulerTensorFlow, MEAD_LAYERS_LR_SCHEDULERS, "sgdr", "lr_scheduler")
+register(CompositeLRSchedulerTensorFlow, MEAD_LAYERS_LR_SCHEDULERS, "composite", "lr_scheduler")
+register(PiecewiseDecaySchedulerTensorFlow, MEAD_LAYERS_LR_SCHEDULERS, "piecewise", "lr_scheduler")
+register(ZarembaDecaySchedulerTensorFlow, MEAD_LAYERS_LR_SCHEDULERS, "zaremba", "lr_scheduler")
+register(CosineDecaySchedulerTensorFlow, MEAD_LAYERS_LR_SCHEDULERS, "cosine", "lr_scheduler")
+register(InverseTimeDecaySchedulerTensorFlow, MEAD_LAYERS_LR_SCHEDULERS, "invtime", "lr_scheduler")
+register(ExponentialDecaySchedulerTensorFlow, MEAD_LAYERS_LR_SCHEDULERS, "exponential", "lr_scheduler")
 
-        def __call__(self, global_step):
-            return self.lr
 
-        def __str__(self):
-            return type(self).__name__ + "()"
-
-    @register_lr_scheduler("warmup_linear")
-    class WarmupLinearSchedulerTensorFlow2(
-        WarmupLearningRateScheduler, tf.keras.optimizers.schedules.LearningRateSchedule
-    ):
-        def __init__(self, **kwargs):
-            lr = float(kwargs.get("lr", kwargs.get("eta", 1.0)))
-            kwargs["lr"] = lr
-            super().__init__(**kwargs)
-
-        def __call__(self, global_step):
-            return tf.minimum(1.0, global_step / float(self.warmup_steps)) * self.lr
-
-        def __str__(self):
-            return type(self).__name__ + "()"
-
-    @register_lr_scheduler("clr")
-    class CyclicLRSchedulerTensorFlow2(LearningRateScheduler, tf.keras.optimizers.schedules.LearningRateSchedule):
-        def __init__(self, max_lr=1e-2, decay_steps=1000, **kwargs):
-            lr = float(kwargs.get("lr", kwargs.get("eta", 1.0)))
-            kwargs["lr"] = lr
-            super().__init__(**kwargs)
-            self.max_lr = max_lr
-            self.decay_steps = decay_steps
-
-        def __call__(self, global_step):
-            gs_f = tf.cast(global_step, tf.float32)
-            cycle = tf.floor(1.0 + gs_f / (2.0 * self.decay_steps))
-            x = tf.abs(gs_f / self.decay_steps - 2.0 * cycle + 1.0)
-            clr = self.lr + (self.max_lr - self.lr) * tf.maximum(0.0, 1.0 - x)
-            return tf.identity(clr, name="lr")
-
-        def __str__(self):
-            return type(self).__name__ + "()"
-
-    @register_lr_scheduler("sgdr")
-    class SGDRSchedulerTensorFlow2(LearningRateScheduler, tf.keras.optimizers.schedules.LearningRateSchedule):
-        def __init__(self, first_decay_steps=1000, **kwargs):
-            super().__init__(**kwargs)
-            self.first_decay_steps = first_decay_steps
-
-        def __call__(self, global_step):
-            return tf.identity(
-                tf.compat.v1.train.cosine_decay_restarts(
-                    self.lr, global_step, first_decay_steps=self.first_decay_steps
-                ),
-                name="lr",
-            )
-
-        def __str__(self):
-            return type(self).__name__ + "()"
-
-    @register_lr_scheduler("zaremba")
-    class ZarembaDecaySchedulerTensorFlow2(tf.keras.optimizers.schedules.PiecewiseConstantDecay):
-        """Utility only, just to simplify the JSON"""
-
-        def __init__(self, boundaries=None, decay_rate=None, **kwargs):
-            lr = float(kwargs.get("lr", kwargs.get("eta", 1.0)))
-            values = [lr / (float(decay_rate) ** i) for i in range(len(boundaries) + 1)]
-            super().__init__(boundaries, values, kwargs.get("name"))
-
-    @register_lr_scheduler("composite")
-    class CompositeLRSchedulerTensorFlow2(CompositeLRScheduler, tf.keras.optimizers.schedules.LearningRateSchedule):
-        pass
-
-    @register_lr_scheduler("piecewise")
-    class PiecewiseConstantDecayTensorFlow2(tf.keras.optimizers.schedules.PiecewiseConstantDecay):
-        def __init__(self, boundaries, values, **kwargs):
-            super().__init__(boundaries, values)
-
-    @register_lr_scheduler("invtime")
-    class InverseTimeDecayTensorFlow2(tf.keras.optimizers.schedules.InverseTimeDecay):
-        def __init__(self, decay_steps=16000, decay_rate=0.05, staircase=False, **kwargs):
-            lr = kwargs.get("lr", kwargs.get("eta", 0.01))
-            super().__init__(lr, decay_steps, decay_rate, staircase, kwargs.get("name"))
-
-    @register_lr_scheduler("exponential")
-    class ExponentialDecayTensorFlow2(tf.keras.optimizers.schedules.ExponentialDecay):
-        def __init__(self, decay_steps=16000, decay_rate=0.5, staircase=False, **kwargs):
-            lr = kwargs.get("lr", kwargs.get("eta", 0.01))
-            super().__init__(lr, decay_steps, decay_rate, staircase, kwargs.get("name"))
-
-    @register_lr_scheduler("cosine")
-    class CosineDecayTensorFlow2(CosineDecayScheduler):
-        def __init__(self, decay_steps=16000, alpha=0.0, **kwargs):
-            kwargs['lr'] = kwargs.get("lr", kwargs.get("eta", 0.01))
-            super().__init__(decay_steps, alpha, **kwargs)
-
-        def __call__(self, global_step):
-            global_step = tf.math.minimum(global_step, self.decay_steps)
-            cosine_decay = 0.5 * (1.0 + tf.cos(3.14159265 * global_step / self.decay_steps))
-            decayed = (1 - self.alpha) * cosine_decay + self.alpha
-            return self.lr * decayed
-
-@export
 class AdamWOptimizer(tf.compat.v1.train.Optimizer):
     """A basic Adam optimizer that includes "correct" L2 weight decay.
 
@@ -585,7 +629,6 @@ def _multiply_gradients(grads_and_vars, gradient_multipliers):
     return multiplied_grads_and_vars
 
 
-@export
 def optimizer(loss_fn, **kwargs):
 
     global_step = tf.compat.v1.train.get_or_create_global_step()
@@ -642,11 +685,9 @@ def optimizer(loss_fn, **kwargs):
     )
 
 
-@export
-class EagerOptimizer(object):
+class EagerOptimizer:
     def __init__(self, loss, optimizer=None, **kwargs):
         self.loss = loss
-        self.global_step = tf.Variable(int(kwargs.get('global_step', 0)))
         if "lr_function" in kwargs:
             lr_function = kwargs["lr_function"]
         else:
@@ -668,7 +709,7 @@ class EagerOptimizer(object):
                 rho = float(kwargs.get("rho", 0.95))
                 eps = float(kwargs.get("epsilon", 1e-6))
                 logger.info("adadelta(eta=%f, rho=%f, epsilon=%f)", lr, rho, eps)
-                self.optimizer = tf.keras.optimizers.Adadelta(lr, rho, eps)
+                self.optimizer = tf.keras.optimizers.Adadelta(lr_function, rho, eps)
             elif optim == "adam":
                 beta1 = float(kwargs.get("beta1", 0.9))
                 beta2 = float(kwargs.get("beta2", 0.999))
@@ -678,15 +719,21 @@ class EagerOptimizer(object):
 
             elif optim == "adamw":
                 import tensorflow_addons as tfa
-
-                wd = float(kwargs.get("weight_decay", 0))
                 beta1 = float(kwargs.get("beta1", 0.9))
                 beta2 = float(kwargs.get("beta2", 0.999))
                 eps = float(kwargs.get("epsilon", 1e-8))
-                logger.info("adamw(eta=%f beta1=%f, beta2=%f, eps=%f, wd=%f)", lr, beta1, beta2, eps, wd)
-                self.optimizer = tfa.optimizers.AdamW(
-                    weight_decay=wd, learning_rate=lr_function, beta_1=beta1, beta_2=beta2, epsilon=eps
-                )
+                wd = float(kwargs.get("weight_decay", 0.0))
+                if wd == 0.0:
+                    logger.info("adam(eta=%f beta1=%f, beta2=%f, eps=%f)", lr, beta1, beta2, eps)
+                    self.optimizer = tf.keras.optimizers.Adam(lr_function, beta1, beta2, eps)
+                else:
+                    def weight_decay_fn():
+                        wd_t = lr_function(tf.cast(self.global_step, tf.float32) / lr) * wd
+                        return wd_t
+                    logger.info("adamw(eta=%f beta1=%f, beta2=%f, eps=%f, wd=%f)", lr, beta1, beta2, eps, wd)
+                    self.optimizer = tfa.optimizers.AdamW(
+                        weight_decay=weight_decay_fn, learning_rate=lr_function, beta_1=beta1, beta_2=beta2, epsilon=eps
+                    )
             elif optim == "rmsprop":
                 # Get mom again with difference default
                 mom = float(kwargs.get("mom", 0.0))
@@ -701,13 +748,16 @@ class EagerOptimizer(object):
 
         logger.info("clip gradients at %s", self.clip)
 
+    @property
+    def global_step(self):
+        return self.optimizer.iterations
+
     def update(self, model, x, y, num_replicas=1):
         with tf.GradientTape() as tape:
             loss_value = self.loss(model, x, y) / num_replicas
         grads = tape.gradient(loss_value, model.trainable_variables)
         grads, _ = tf.clip_by_global_norm(grads, self.clip)
         self.optimizer.apply_gradients(zip(grads, model.trainable_variables))
-        self.global_step.assign_add(1)
         return loss_value
 
     def update_with_hidden(self, model, h, x, y):
@@ -717,5 +767,4 @@ class EagerOptimizer(object):
         grads = tape.gradient(loss_value, model.trainable_variables)
         grads, _ = tf.clip_by_global_norm(grads, self.clip)
         self.optimizer.apply_gradients(zip(grads, model.trainable_variables))
-        self.global_step.assign_add(1)
         return loss_value, h
