@@ -11,7 +11,7 @@ from itertools import chain
 from binascii import hexlify
 from functools import partial
 from collections import Counter
-from typing import List, Tuple, Union, Optional, Dict, Any, Set, Pattern
+from typing import List, Tuple, Union, Optional, Dict, Any, Set, Pattern, TextIO
 from functools import partial, update_wrapper, wraps
 import numpy as np
 from six.moves.urllib.request import urlretrieve
@@ -859,6 +859,13 @@ def read_conll(f, doc_pattern=None, delim=None, metadata=False, allow_comments=F
                 yield x
 
 
+@export
+@str_file(wf="w")
+def write_conll(wf: Union[str, TextIO], sentences: List[List[List[str]]], delim: Optional[str] = None) -> None:
+    delim = " " if delim is None else delim
+    wf.write("\n\n".join(["\n".join([delim.join(row) for row in sentence]) for sentence in sentences]) + "\n\n")
+
+
 @export
 @str_file
 def read_conll_sentences(f, delim=None, allow_comments=True, comment_pattern="#"):
@@ -1451,6 +1458,14 @@ def get_env_gpus() -> List[str]:
     return os.getenv("CUDA_VISIBLE_DEVICES", os.getenv("NVIDIA_VISIBLE_DEVICES", os.getenv("NV_GPU", "0"))).split(",")
 
 
+@export
+def get_num_gpus_multiworker() -> int:
+    """Get the number of GPUs in multi-worker distributed training
+
+    :return:
+    """
+    return int(os.environ.get("WORLD_SIZE", 1))
+
 @export
 def ngrams(sentence: List[str], filtsz: int = 3, joiner: str = "@@") -> List[str]:
     """Generate ngrams over a sentence
@@ -1586,6 +1601,28 @@ def to_numpy(x):
     return x.numpy()
 
 
+@export
+def mlm_masking(inputs, mask_value, vocab_size, ignore_prefix, ignore_suffix):
+    labels = np.copy(inputs)
+    masked_indices = np.random.binomial(size=len(inputs), n=1, p=0.15)
+    masked_indices[np.random.randint(1, len(inputs)-1)] = 1
+    if ignore_prefix:
+        masked_indices[0] = 0
+    if ignore_suffix:
+        masked_indices[-1] = 0
+    # Anything not masked is 0 so no loss
+    labels[masked_indices == 0] = 0
+    # Of the masked items, mask 80% of them with [MASK]
+    indices_replaced = np.random.binomial(size=len(inputs), n=1, p=0.8)
+    indices_replaced = indices_replaced & masked_indices
+    inputs[indices_replaced == 1] = mask_value
+    indices_random = np.random.binomial(size=len(inputs), n=1, p=0.5)
+    # Replace 10% of them with random words, rest preserved for auto-encoding
+    indices_random = indices_random & masked_indices & ~indices_replaced
+    random_words = np.random.randint(low=len(Offsets.VALUES) + 3, high=vocab_size-1, size=len(inputs))
+    inputs[indices_random == 1] = random_words[indices_random == 1]
+    return inputs, labels
+
 # https://stackoverflow.com/questions/5909873/how-can-i-pretty-print-ascii-tables-with-python
 @export
 def print_table(rows: collections.namedtuple, columns: Optional[Set[str]] = None) -> None:
@@ -1622,3 +1659,24 @@ def print_table(rows: collections.namedtuple, columns: Optional[Set[str]] = None
         for field in fields:
             print("%*s = %s" % (hwidth, field, getattr(row, field)))
 
+
+@export
+class Average:
+    def __init__(self, name, fmt=':f'):
+        self.name = name
+        self.fmt = fmt
+        self.val = 0
+        self.avg = 0
+        self.sum = 0
+        self.count = 0
+
+    def update(self, val, n=1):
+        self.val = val
+        self.sum += val * n
+        self.count += n
+        self.avg = self.sum / self.count
+
+    def __str__(self):
+        fmtstr = '{name} {val' + self.fmt + '} ({avg' + self.fmt + '})'
+        return fmtstr.format(**self.__dict__)
+
